@@ -16,18 +16,20 @@
 import base64
 import hmac
 import time
-import sys
-
 from hashlib import sha1
 
+import libcloud.utils.py3
+
 try:
-    from lxml.etree import Element, SubElement
+    if libcloud.utils.py3.DEFAULT_LXML:
+        from lxml.etree import Element, SubElement
+    else:
+        from xml.etree.ElementTree import Element, SubElement
 except ImportError:
     from xml.etree.ElementTree import Element, SubElement
 
 from libcloud.utils.py3 import httplib
 from libcloud.utils.py3 import urlquote
-from libcloud.utils.py3 import urlencode
 from libcloud.utils.py3 import b
 from libcloud.utils.py3 import tostring
 
@@ -46,20 +48,28 @@ from libcloud.storage.types import ContainerDoesNotExistError
 from libcloud.storage.types import ObjectDoesNotExistError
 from libcloud.storage.types import ObjectHashMismatchError
 
-
 # How long before the token expires
 EXPIRATION_SECONDS = 15 * 60
 
 S3_US_STANDARD_HOST = 's3.amazonaws.com'
+S3_US_EAST2_HOST = 's3-us-east-2.amazonaws.com'
 S3_US_WEST_HOST = 's3-us-west-1.amazonaws.com'
 S3_US_WEST_OREGON_HOST = 's3-us-west-2.amazonaws.com'
+S3_US_GOV_WEST_HOST = 's3-us-gov-west-1.amazonaws.com'
 S3_CN_NORTH_HOST = 's3.cn-north-1.amazonaws.com.cn'
+S3_CN_NORTHWEST_HOST = 's3.cn-northwest-1.amazonaws.com.cn'
 S3_EU_WEST_HOST = 's3-eu-west-1.amazonaws.com'
+S3_EU_WEST2_HOST = 's3-eu-west-2.amazonaws.com'
+S3_EU_CENTRAL_HOST = 's3-eu-central-1.amazonaws.com'
+S3_AP_SOUTH_HOST = 's3-ap-south-1.amazonaws.com'
 S3_AP_SOUTHEAST_HOST = 's3-ap-southeast-1.amazonaws.com'
+S3_AP_SOUTHEAST2_HOST = 's3-ap-southeast-2.amazonaws.com'
 S3_AP_NORTHEAST1_HOST = 's3-ap-northeast-1.amazonaws.com'
 S3_AP_NORTHEAST2_HOST = 's3-ap-northeast-2.amazonaws.com'
 S3_AP_NORTHEAST_HOST = S3_AP_NORTHEAST1_HOST
 S3_SA_EAST_HOST = 's3-sa-east-1.amazonaws.com'
+S3_SA_SOUTHEAST2_HOST = 's3-sa-east-2.amazonaws.com'
+S3_CA_CENTRAL_HOST = 's3-ca-central-1.amazonaws.com'
 
 API_VERSION = '2006-03-01'
 NAMESPACE = 'http://s3.amazonaws.com/doc/%s/' % (API_VERSION)
@@ -79,7 +89,7 @@ class S3Response(AWSBaseResponse):
 
     def success(self):
         i = int(self.status)
-        return i >= 200 and i <= 299 or i in self.valid_response_codes
+        return 200 <= i <= 299 or i in self.valid_response_codes
 
     def parse_error(self):
         if self.status in [httplib.UNAUTHORIZED, httplib.FORBIDDEN]:
@@ -175,6 +185,20 @@ class S3Connection(AWSTokenConnection, BaseS3Connection):
     pass
 
 
+class S3SignatureV4Connection(SignedAWSConnection, BaseS3Connection):
+    service_name = 's3'
+    version = API_VERSION
+
+    def __init__(self, user_id, key, secure=True, host=None, port=None,
+                 url=None, timeout=None, proxy_url=None, token=None,
+                 retry_delay=None, backoff=None):
+        super(S3SignatureV4Connection, self).__init__(
+            user_id, key, secure, host,
+            port, url, timeout, proxy_url,
+            token, retry_delay, backoff,
+            4)  # force version 4
+
+
 class S3MultipartUpload(object):
     """
     Class representing an amazon s3 multipart upload
@@ -412,13 +436,15 @@ class BaseS3StorageDriver(StorageDriver):
 
     def download_object_as_stream(self, obj, chunk_size=None):
         obj_path = self._get_object_path(obj.container, obj.name)
-        response = self.connection.request(obj_path, method='GET', raw=True)
+        response = self.connection.request(obj_path, method='GET',
+                                           stream=True, raw=True)
 
-        return self._get_object(obj=obj, callback=read_in_chunks,
-                                response=response,
-                                callback_kwargs={'iterator': response.response,
-                                                 'chunk_size': chunk_size},
-                                success_status_code=httplib.OK)
+        return self._get_object(
+            obj=obj, callback=read_in_chunks,
+            response=response,
+            callback_kwargs={'iterator': response.iter_content(CHUNK_SIZE),
+                             'chunk_size': chunk_size},
+            success_status_code=httplib.OK)
 
     def upload_object(self, file_path, container, object_name, extra=None,
                       verify_hash=True, ex_storage_class=None):
@@ -428,94 +454,65 @@ class BaseS3StorageDriver(StorageDriver):
         :param ex_storage_class: Storage class
         :type ex_storage_class: ``str``
         """
-        upload_func = self._upload_file
-        upload_func_kwargs = {'file_path': file_path}
-
         return self._put_object(container=container, object_name=object_name,
-                                upload_func=upload_func,
-                                upload_func_kwargs=upload_func_kwargs,
                                 extra=extra, file_path=file_path,
                                 verify_hash=verify_hash,
                                 storage_class=ex_storage_class)
 
-    def _upload_multipart(self, response, data, iterator, container,
-                          object_name, calculate_hash=True):
+    def _initiate_multipart(self, container, object_name, headers=None):
         """
-        Callback invoked for uploading data to S3 using Amazon's
-        multipart upload mechanism
+        Initiates a multipart upload to S3
 
-        :param response: Response object from the initial POST request
-        :type response: :class:`S3RawResponse`
-
-        :param data: Any data from the initial POST request
-        :type data: ``str``
-
-        :param iterator: The generator for fetching the upload data
-        :type iterator: ``generator``
-
-        :param container: The container owning the object to which data is
-            being uploaded
+        :param container: The destination container
         :type container: :class:`Container`
 
-        :param object_name: The name of the object to which we are uploading
+        :param object_name: The name of the object which we are uploading
         :type object_name: ``str``
 
-        :keyword calculate_hash: Indicates if we must calculate the data hash
-        :type calculate_hash: ``bool``
+        :keyword headers: Additional headers to send with the request
+        :type headers: ``dict``
 
-        :return: A tuple of (status, checksum, bytes transferred)
-        :rtype: ``tuple``
+        :return: The id of the newly created multipart upload
+        :rtype: ``str``
         """
+        headers = headers or {}
 
-        object_path = self._get_object_path(container, object_name)
+        request_path = self._get_object_path(container, object_name)
+        params = {'uploads': ''}
 
-        # Get the upload id from the response xml
-        response.body = response.response.read()
-        body = response.parse_body()
-        upload_id = body.find(fixxpath(xpath='UploadId',
-                                       namespace=self.namespace)).text
+        response = self.connection.request(request_path, method='POST',
+                                           headers=headers, params=params)
 
-        try:
-            # Upload the data through the iterator
-            result = self._upload_from_iterator(iterator, object_path,
-                                                upload_id, calculate_hash)
-            (chunks, data_hash, bytes_transferred) = result
+        if response.status != httplib.OK:
+            raise LibcloudError('Error initiating multipart upload',
+                                driver=self)
 
-            # Commit the chunk info and complete the upload
-            etag = self._commit_multipart(object_path, upload_id, chunks)
-        except Exception:
-            exc = sys.exc_info()[1]
-            # Amazon provides a mechanism for aborting an upload.
-            self._abort_multipart(object_path, upload_id)
-            raise exc
+        return findtext(element=response.object, xpath='UploadId',
+                        namespace=self.namespace)
 
-        # Modify the response header of the first request. This is used
-        # by other functions once the callback is done
-        response.headers['etag'] = etag
-
-        return (True, data_hash, bytes_transferred)
-
-    def _upload_from_iterator(self, iterator, object_path, upload_id,
-                              calculate_hash=True):
+    def _upload_multipart_chunks(self, container, object_name, upload_id,
+                                 stream, calculate_hash=True):
         """
         Uploads data from an iterator in fixed sized chunks to S3
 
-        :param iterator: The generator for fetching the upload data
-        :type iterator: ``generator``
+        :param container: The destination container
+        :type container: :class:`Container`
 
-        :param object_path: The path of the object to which we are uploading
+        :param object_name: The name of the object which we are uploading
         :type object_name: ``str``
 
         :param upload_id: The upload id allocated for this multipart upload
         :type upload_id: ``str``
 
+        :param stream: The generator for fetching the upload data
+        :type stream: ``generator``
+
         :keyword calculate_hash: Indicates if we must calculate the data hash
         :type calculate_hash: ``bool``
 
         :return: A tuple of (chunk info, checksum, bytes transferred)
         :rtype: ``tuple``
         """
-
         data_hash = None
         if calculate_hash:
             data_hash = self._get_hash_function()
@@ -525,8 +522,10 @@ class BaseS3StorageDriver(StorageDriver):
         chunks = []
         params = {'uploadId': upload_id}
 
+        request_path = self._get_object_path(container, object_name)
+
         # Read the input data in chunk sizes suitable for AWS
-        for data in read_in_chunks(iterator, chunk_size=CHUNK_SIZE,
+        for data in read_in_chunks(stream, chunk_size=CHUNK_SIZE,
                                    fill_size=True, yield_empty=True):
             bytes_transferred += len(data)
 
@@ -537,20 +536,23 @@ class BaseS3StorageDriver(StorageDriver):
             chunk_hash.update(data)
             chunk_hash = base64.b64encode(chunk_hash.digest()).decode('utf-8')
 
-            # This provides an extra level of data check and is recommended
-            # by amazon
-            headers = {'Content-MD5': chunk_hash}
+            # The Content-MD5 header provides an extra level of data check and
+            # is recommended by amazon
+            headers = {
+                'Content-Length': len(data),
+                'Content-MD5': chunk_hash,
+            }
+
             params['partNumber'] = count
 
-            request_path = '?'.join((object_path, urlencode(params)))
-
             resp = self.connection.request(request_path, method='PUT',
-                                           data=data, headers=headers)
+                                           data=data, headers=headers,
+                                           params=params)
 
             if resp.status != httplib.OK:
                 raise LibcloudError('Error uploading chunk', driver=self)
 
-            server_hash = resp.headers['etag']
+            server_hash = resp.headers['etag'].replace('"', '')
 
             # Keep this data for a later commit
             chunks.append((count, server_hash))
@@ -561,20 +563,25 @@ class BaseS3StorageDriver(StorageDriver):
 
         return (chunks, data_hash, bytes_transferred)
 
-    def _commit_multipart(self, object_path, upload_id, chunks):
+    def _commit_multipart(self, container, object_name, upload_id, chunks):
         """
         Makes a final commit of the data.
 
-        :param object_path: Server side object path.
-        :type object_path: ``str``
+        :param container: The destination container
+        :type container: :class:`Container`
 
-        :param upload_id: ID of the multipart upload.
+        :param object_name: The name of the object which we are uploading
+        :type object_name: ``str``
+
+        :param upload_id: The upload id allocated for this multipart upload
         :type upload_id: ``str``
 
-        :param upload_id: A list of (chunk_number, chunk_hash) tuples.
-        :type upload_id: ``list``
-        """
+        :param chunks: A list of (chunk_number, chunk_hash) tuples.
+        :type chunks: ``list``
 
+        :return: The server side hash of the uploaded data
+        :rtype: ``str``
+        """
         root = Element('CompleteMultipartUpload')
 
         for (count, etag) in chunks:
@@ -587,9 +594,11 @@ class BaseS3StorageDriver(StorageDriver):
 
         data = tostring(root)
 
+        headers = {'Content-Length': len(data)}
         params = {'uploadId': upload_id}
-        request_path = '?'.join((object_path, urlencode(params)))
-        response = self.connection.request(request_path, data=data,
+        request_path = self._get_object_path(container, object_name)
+        response = self.connection.request(request_path, headers=headers,
+                                           params=params, data=data,
                                            method='POST')
 
         if response.status != httplib.OK:
@@ -605,20 +614,25 @@ class BaseS3StorageDriver(StorageDriver):
                                          namespace=self.namespace)).text
         return server_hash
 
-    def _abort_multipart(self, object_path, upload_id):
+    def _abort_multipart(self, container, object_name, upload_id):
         """
         Aborts an already initiated multipart upload
 
-        :param object_path: Server side object path.
-        :type object_path: ``str``
+        :param container: The destination container
+        :type container: :class:`Container`
 
-        :param upload_id: ID of the multipart upload.
+        :param object_name: The name of the object which we are uploading
+        :type object_name: ``str``
+
+        :param upload_id: The upload id allocated for this multipart upload
         :type upload_id: ``str``
         """
 
         params = {'uploadId': upload_id}
-        request_path = '?'.join((object_path, urlencode(params)))
-        resp = self.connection.request(request_path, method='DELETE')
+        request_path = self._get_object_path(container, object_name)
+
+        resp = self.connection.request(request_path, method='DELETE',
+                                       params=params)
 
         if resp.status != httplib.NO_CONTENT:
             raise LibcloudError('Error in multipart abort. status_code=%d' %
@@ -640,29 +654,15 @@ class BaseS3StorageDriver(StorageDriver):
         # Amazon provides a different (complex?) mechanism to do multipart
         # uploads
         if self.supports_s3_multipart_upload:
-            # Initiate the multipart request and get an upload id
-            upload_func = self._upload_multipart
-            upload_func_kwargs = {'iterator': iterator,
-                                  'container': container,
-                                  'object_name': object_name}
-            method = 'POST'
-            iterator = iter('')
-            params = 'uploads'
-
-        elif self.supports_chunked_encoding:
-            upload_func = self._stream_data
-            upload_func_kwargs = {'iterator': iterator}
-        else:
-            # In this case, we have to load the entire object to
-            # memory and send it as normal data
-            upload_func = self._upload_data
-            upload_func_kwargs = {}
-
+            return self._put_object_multipart(container=container,
+                                              object_name=object_name,
+                                              extra=extra,
+                                              stream=iterator,
+                                              verify_hash=False,
+                                              storage_class=ex_storage_class)
         return self._put_object(container=container, object_name=object_name,
-                                upload_func=upload_func,
-                                upload_func_kwargs=upload_func_kwargs,
                                 extra=extra, method=method, query_args=params,
-                                iterator=iterator, verify_hash=False,
+                                stream=iterator, verify_hash=False,
                                 storage_class=ex_storage_class)
 
     def delete_object(self, obj):
@@ -702,8 +702,8 @@ class BaseS3StorageDriver(StorageDriver):
             raise LibcloudError('Feature not supported', driver=self)
 
         # Get the data for a specific container
-        request_path = '%s/?uploads' % (self._get_container_path(container))
-        params = {'max-uploads': RESPONSES_PER_REQUEST}
+        request_path = self._get_container_path(container)
+        params = {'max-uploads': RESPONSES_PER_REQUEST, 'uploads': ''}
 
         if prefix:
             params['prefix'] = prefix
@@ -727,7 +727,6 @@ class BaseS3StorageDriver(StorageDriver):
             # pylint: disable=maybe-no-member
             for node in body.findall(fixxpath(xpath='Upload',
                                               namespace=self.namespace)):
-
                 initiator = node.find(fixxpath(xpath='Initiator',
                                                namespace=self.namespace))
                 owner = node.find(fixxpath(xpath='Owner',
@@ -774,26 +773,19 @@ class BaseS3StorageDriver(StorageDriver):
         # Iterate through the container and delete the upload ids
         for upload in self.ex_iterate_multipart_uploads(container, prefix,
                                                         delimiter=None):
-            object_path = '/%s/%s' % (container.name, upload.key)
-            self._abort_multipart(object_path, upload.id)
+            self._abort_multipart(container, upload.key, upload.id)
 
     def _clean_object_name(self, name):
         name = urlquote(name)
         return name
 
-    def _put_object(self, container, object_name, upload_func,
-                    upload_func_kwargs, method='PUT', query_args=None,
-                    extra=None, file_path=None, iterator=None,
-                    verify_hash=True, storage_class=None):
+    def _put_object(self, container, object_name, method='PUT',
+                    query_args=None, extra=None, file_path=None,
+                    stream=None, verify_hash=True, storage_class=None):
         headers = {}
         extra = extra or {}
-        storage_class = storage_class or 'standard'
-        if storage_class not in ['standard', 'reduced_redundancy']:
-            raise ValueError(
-                'Invalid storage class value: %s' % (storage_class))
 
-        key = self.http_vendor_prefix + '-storage-class'
-        headers[key] = storage_class.upper()
+        headers.update(self._to_storage_class_headers(storage_class))
 
         content_type = extra.get('content_type', None)
         meta_data = extra.get('meta_data', None)
@@ -812,25 +804,21 @@ class BaseS3StorageDriver(StorageDriver):
         if query_args:
             request_path = '?'.join((request_path, query_args))
 
-        # TODO: Let the underlying exceptions bubble up and capture the SIGPIPE
-        # here.
-        # SIGPIPE is thrown if the provided container does not exist or the
-        # user does not have correct permission
         result_dict = self._upload_object(
             object_name=object_name, content_type=content_type,
-            upload_func=upload_func, upload_func_kwargs=upload_func_kwargs,
             request_path=request_path, request_method=method,
-            headers=headers, file_path=file_path, iterator=iterator)
+            headers=headers, file_path=file_path, stream=stream)
 
         response = result_dict['response']
         bytes_transferred = result_dict['bytes_transferred']
         headers = response.headers
-        response = response.response
+        response = response
         server_hash = headers.get('etag', '').replace('"', '')
 
         if (verify_hash and result_dict['data_hash'] != server_hash):
             raise ObjectHashMismatchError(
-                value='MD5 hash checksum does not match',
+                value='MD5 hash {0} checksum does not match {1}'.format(
+                    server_hash, result_dict['data_hash']),
                 object_name=object_name, driver=self)
         elif response.status == httplib.OK:
             obj = Object(
@@ -844,6 +832,99 @@ class BaseS3StorageDriver(StorageDriver):
                 'Unexpected status code, status_code=%s' % (response.status),
                 driver=self)
 
+    def _put_object_multipart(self, container, object_name, stream,
+                              extra=None, verify_hash=False,
+                              storage_class=None):
+        """
+        Uploads an object using the S3 multipart algorithm.
+
+        :param container: The destination container
+        :type container: :class:`Container`
+
+        :param object_name: The name of the object which we are uploading
+        :type object_name: ``str``
+
+        :param stream: The generator for fetching the upload data
+        :type stream: ``generator``
+
+        :keyword verify_hash: Indicates if we must calculate the data hash
+        :type verify_hash: ``bool``
+
+        :keyword extra: Additional options
+        :type extra: ``dict``
+
+        :keyword storage_class: The name of the S3 object's storage class
+        :type extra: ``str``
+
+        :return: The uploaded object
+        :rtype: :class:`Object`
+        """
+        headers = {}
+        extra = extra or {}
+
+        headers.update(self._to_storage_class_headers(storage_class))
+
+        content_type = extra.get('content_type', None)
+        meta_data = extra.get('meta_data', None)
+        acl = extra.get('acl', None)
+
+        if not content_type:
+            content_type, _ = libcloud.utils.files.guess_file_mime_type(
+                object_name)
+
+        if content_type:
+            headers['Content-Type'] = content_type
+
+        if meta_data:
+            for key, value in list(meta_data.items()):
+                key = self.http_vendor_prefix + '-meta-%s' % (key)
+                headers[key] = value
+
+        if acl:
+            headers[self.http_vendor_prefix + '-acl'] = acl
+
+        upload_id = self._initiate_multipart(container, object_name,
+                                             headers=headers)
+
+        try:
+            result = self._upload_multipart_chunks(container, object_name,
+                                                   upload_id, stream,
+                                                   calculate_hash=verify_hash)
+            chunks, data_hash, bytes_transferred = result
+
+            # Commit the chunk info and complete the upload
+            etag = self._commit_multipart(container, object_name, upload_id,
+                                          chunks)
+        except Exception:
+            # Amazon provides a mechanism for aborting an upload.
+            self._abort_multipart(container, object_name, upload_id)
+            raise
+
+        return Object(
+            name=object_name, size=bytes_transferred, hash=etag,
+            extra={'acl': acl}, meta_data=meta_data, container=container,
+            driver=self)
+
+    def _to_storage_class_headers(self, storage_class):
+        """
+        Generates request headers given a storage class name.
+
+        :keyword storage_class: The name of the S3 object's storage class
+        :type extra: ``str``
+
+        :return: Headers to include in a request
+        :rtype: :dict:
+        """
+        headers = {}
+        storage_class = storage_class or 'standard'
+        if storage_class not in ['standard', 'reduced_redundancy']:
+            raise ValueError(
+                'Invalid storage class value: %s' % (storage_class))
+
+        key = self.http_vendor_prefix + '-storage-class'
+        headers[key] = storage_class.upper()
+        return headers
+
     def _to_containers(self, obj, xpath):
         for element in obj.findall(fixxpath(xpath=xpath,
                                             namespace=self.namespace)):
@@ -919,10 +1000,23 @@ class BaseS3StorageDriver(StorageDriver):
 
 
 class S3StorageDriver(AWSDriver, BaseS3StorageDriver):
-    connectionCls = S3Connection
+    name = 'Amazon S3 (us-east-1)'
+    connectionCls = S3SignatureV4Connection
+    region_name = 'us-east-1'
 
 
-class S3USWestConnection(S3Connection):
+class S3USEast2Connection(S3SignatureV4Connection):
+    host = S3_US_EAST2_HOST
+
+
+class S3USEast2StorageDriver(S3StorageDriver):
+    name = 'Amazon S3 (us-east-2)'
+    connectionCls = S3USEast2Connection
+    ex_location_name = 'us-east-2'
+    region_name = 'us-east-2'
+
+
+class S3USWestConnection(S3SignatureV4Connection):
     host = S3_US_WEST_HOST
 
 
@@ -930,9 +1024,10 @@ class S3USWestStorageDriver(S3StorageDriver):
     name = 'Amazon S3 (us-west-1)'
     connectionCls = S3USWestConnection
     ex_location_name = 'us-west-1'
+    region_name = 'us-west-1'
 
 
-class S3USWestOregonConnection(S3Connection):
+class S3USWestOregonConnection(S3SignatureV4Connection):
     host = S3_US_WEST_OREGON_HOST
 
 
@@ -940,21 +1035,33 @@ class S3USWestOregonStorageDriver(S3StorageDriver):
     name = 'Amazon S3 (us-west-2)'
     connectionCls = S3USWestOregonConnection
     ex_location_name = 'us-west-2'
+    region_name = 'us-west-2'
 
 
-class S3CNNorthConnection(SignedAWSConnection, BaseS3Connection):
+class S3USGovWestConnection(S3SignatureV4Connection):
+    host = S3_US_GOV_WEST_HOST
+
+
+class S3USGovWestStorageDriver(S3StorageDriver):
+    name = 'Amazon S3 (us-gov-west-1)'
+    connectionCls = S3USGovWestConnection
+    ex_location_name = 'us-gov-west-1'
+    region_name = 'us-gov-west-1'
+
+
+class S3CNNorthWestConnection(S3SignatureV4Connection):
+    host = S3_CN_NORTHWEST_HOST
+
+
+class S3CNNorthWestStorageDriver(S3StorageDriver):
+    name = 'Amazon S3 (cn-northwest-1)'
+    connectionCls = S3CNNorthWestConnection
+    ex_location_name = 'cn-northwest-1'
+    region_name = 'cn-northwest-1'
+
+
+class S3CNNorthConnection(S3SignatureV4Connection):
     host = S3_CN_NORTH_HOST
-    service_name = 's3'
-    version = API_VERSION
-
-    def __init__(self, user_id, key, secure=True, host=None, port=None,
-                 url=None, timeout=None, proxy_url=None, token=None,
-                 retry_delay=None, backoff=None):
-        super(S3CNNorthConnection, self).__init__(
-            user_id, key, secure, host,
-            port, url, timeout, proxy_url,
-            token, retry_delay, backoff,
-            4)  # force version 4
 
 
 class S3CNNorthStorageDriver(S3StorageDriver):
@@ -962,11 +1069,9 @@ class S3CNNorthStorageDriver(S3StorageDriver):
     connectionCls = S3CNNorthConnection
     ex_location_name = 'cn-north-1'
     region_name = 'cn-north-1'
-    # v4 auth and multipart_upload currently do not work.
-    supports_s3_multipart_upload = False
 
 
-class S3EUWestConnection(S3Connection):
+class S3EUWestConnection(S3SignatureV4Connection):
     host = S3_EU_WEST_HOST
 
 
@@ -974,9 +1079,32 @@ class S3EUWestStorageDriver(S3StorageDriver):
     name = 'Amazon S3 (eu-west-1)'
     connectionCls = S3EUWestConnection
     ex_location_name = 'EU'
+    region_name = 'eu-west-1'
 
 
-class S3APSEConnection(S3Connection):
+class S3EUWest2Connection(S3SignatureV4Connection):
+    host = S3_EU_WEST2_HOST
+
+
+class S3EUWest2StorageDriver(S3StorageDriver):
+    name = 'Amazon S3 (eu-west-2)'
+    connectionCls = S3EUWest2Connection
+    ex_location_name = 'eu-west-2'
+    region_name = 'eu-west-2'
+
+
+class S3EUCentralConnection(S3SignatureV4Connection):
+    host = S3_EU_CENTRAL_HOST
+
+
+class S3EUCentralStorageDriver(S3StorageDriver):
+    name = 'Amazon S3 (eu-central-1)'
+    connectionCls = S3EUCentralConnection
+    ex_location_name = 'eu-central-1'
+    region_name = 'eu-central-1'
+
+
+class S3APSEConnection(S3SignatureV4Connection):
     host = S3_AP_SOUTHEAST_HOST
 
 
@@ -984,11 +1112,24 @@ class S3APSEStorageDriver(S3StorageDriver):
     name = 'Amazon S3 (ap-southeast-1)'
     connectionCls = S3APSEConnection
     ex_location_name = 'ap-southeast-1'
+    region_name = 'ap-southeast-1'
 
 
-class S3APNE1Connection(S3Connection):
+class S3APSE2Connection(S3SignatureV4Connection):
+    host = S3_AP_SOUTHEAST2_HOST
+
+
+class S3APSE2StorageDriver(S3StorageDriver):
+    name = 'Amazon S3 (ap-southeast-2)'
+    connectionCls = S3APSE2Connection
+    ex_location_name = 'ap-southeast-2'
+    region_name = 'ap-southeast-2'
+
+
+class S3APNE1Connection(S3SignatureV4Connection):
     host = S3_AP_NORTHEAST1_HOST
 
+
 S3APNEConnection = S3APNE1Connection
 
 
@@ -996,22 +1137,14 @@ class S3APNE1StorageDriver(S3StorageDriver):
     name = 'Amazon S3 (ap-northeast-1)'
     connectionCls = S3APNEConnection
     ex_location_name = 'ap-northeast-1'
+    region_name = 'ap-northeast-1'
+
 
 S3APNEStorageDriver = S3APNE1StorageDriver
 
 
-class S3APNE2Connection(SignedAWSConnection, BaseS3Connection):
+class S3APNE2Connection(S3SignatureV4Connection):
     host = S3_AP_NORTHEAST2_HOST
-    service_name = 's3'
-    version = API_VERSION
-
-    def __init__(self, user_id, key, secure=True, host=None, port=None,
-                 url=None, timeout=None, proxy_url=None, token=None,
-                 retry_delay=None, backoff=None):
-        super(S3APNE2Connection, self).__init__(user_id, key, secure, host,
-                                                port, url, timeout, proxy_url,
-                                                token, retry_delay, backoff,
-                                                4)  # force version 4
 
 
 class S3APNE2StorageDriver(S3StorageDriver):
@@ -1019,11 +1152,20 @@ class S3APNE2StorageDriver(S3StorageDriver):
     connectionCls = S3APNE2Connection
     ex_location_name = 'ap-northeast-2'
     region_name = 'ap-northeast-2'
-    # v4 auth and multipart_upload currently do not work.
-    supports_s3_multipart_upload = False
 
 
-class S3SAEastConnection(S3Connection):
+class S3APSouthConnection(S3SignatureV4Connection):
+    host = S3_AP_SOUTH_HOST
+
+
+class S3APSouthStorageDriver(S3StorageDriver):
+    name = 'Amazon S3 (ap-south-1)'
+    connectionCls = S3APSouthConnection
+    ex_location_name = 'ap-south-1'
+    region_name = 'ap-south-1'
+
+
+class S3SAEastConnection(S3SignatureV4Connection):
     host = S3_SA_EAST_HOST
 
 
@@ -1031,3 +1173,15 @@ class S3SAEastStorageDriver(S3StorageDriver):
     name = 'Amazon S3 (sa-east-1)'
     connectionCls = S3SAEastConnection
     ex_location_name = 'sa-east-1'
+    region_name = 'sa-east-1'
+
+
+class S3CACentralConnection(S3SignatureV4Connection):
+    host = S3_CA_CENTRAL_HOST
+
+
+class S3CACentralStorageDriver(S3StorageDriver):
+    name = 'Amazon S3 (ca-central-1)'
+    connectionCls = S3CACentralConnection
+    ex_location_name = 'ca-central-1'
+    region_name = 'ca-central-1'
