@@ -30,7 +30,7 @@ from libcloud.common.digitalocean import DigitalOcean_v1_Error
 from libcloud.compute.base import NodeImage
 from libcloud.compute.drivers.digitalocean import DigitalOceanNodeDriver
 
-from libcloud.test import LibcloudTestCase, MockHttpTestCase
+from libcloud.test import LibcloudTestCase, MockHttp
 from libcloud.test.file_fixtures import ComputeFileFixtures
 from libcloud.test.secrets import DIGITALOCEAN_v1_PARAMS
 from libcloud.test.secrets import DIGITALOCEAN_v2_PARAMS
@@ -40,8 +40,8 @@ from libcloud.test.secrets import DIGITALOCEAN_v2_PARAMS
 class DigitalOcean_v2_Tests(LibcloudTestCase):
 
     def setUp(self):
-        DigitalOceanNodeDriver.connectionCls.conn_classes = \
-            (None, DigitalOceanMockHttp)
+        DigitalOceanNodeDriver.connectionCls.conn_class = \
+            DigitalOceanMockHttp
         DigitalOceanMockHttp.type = None
         self.driver = DigitalOceanNodeDriver(*DIGITALOCEAN_v2_PARAMS)
 
@@ -94,6 +94,7 @@ class DigitalOcean_v2_Tests(LibcloudTestCase):
         self.assertEqual(nodes[0].public_ips, ['104.236.32.182'])
         self.assertEqual(nodes[0].extra['image']['id'], 6918990)
         self.assertEqual(nodes[0].extra['size_slug'], '512mb')
+        self.assertEqual(len(nodes[0].extra['tags']), 2)
 
     def test_list_nodes_fills_created_datetime(self):
         nodes = self.driver.list_nodes()
@@ -148,12 +149,43 @@ class DigitalOcean_v2_Tests(LibcloudTestCase):
         result = self.driver.ex_shutdown_node(node)
         self.assertTrue(result)
 
+    def test_ex_hard_reboot_success(self):
+        node = self.driver.list_nodes()[0]
+        DigitalOceanMockHttp.type = 'POWERCYCLE'
+        result = self.driver.ex_hard_reboot(node)
+        self.assertTrue(result)
+
+    def test_ex_rebuild_node_success(self):
+        node = self.driver.list_nodes()[0]
+        DigitalOceanMockHttp.type = 'REBUILD'
+        result = self.driver.ex_rebuild_node(node)
+        self.assertTrue(result)
+
+    def test_ex_resize_node_success(self):
+        node = self.driver.list_nodes()[0]
+        size = self.driver.list_sizes()[0]
+        DigitalOceanMockHttp.type = 'RESIZE'
+        result = self.driver.ex_resize_node(node, size)
+        self.assertTrue(result)
+
     def test_destroy_node_success(self):
         node = self.driver.list_nodes()[0]
         DigitalOceanMockHttp.type = 'DESTROY'
         result = self.driver.destroy_node(node)
         self.assertTrue(result)
 
+    def test_ex_change_kernel_success(self):
+        node = self.driver.list_nodes()[0]
+        DigitalOceanMockHttp.type = 'KERNELCHANGE'
+        result = self.driver.ex_change_kernel(node, 7515)
+        self.assertTrue(result)
+
+    def test_ex_enable_ipv6_success(self):
+        node = self.driver.list_nodes()[0]
+        DigitalOceanMockHttp.type = 'ENABLEIPV6'
+        result = self.driver.ex_enable_ipv6(node)
+        self.assertTrue(result)
+
     def test_ex_rename_node_success(self):
         node = self.driver.list_nodes()[0]
         DigitalOceanMockHttp.type = 'RENAME'
@@ -236,8 +268,105 @@ class DigitalOcean_v2_Tests(LibcloudTestCase):
         resp = self.driver.destroy_volume(volume)
         self.assertTrue(resp)
 
+    def test_list_volume_snapshots(self):
+        volume = self.driver.list_volumes()[0]
+        snapshots = self.driver.list_volume_snapshots(volume)
+        self.assertEqual(len(snapshots), 3)
+        snapshot1, snapshot2, snapshot3 = snapshots
+        self.assertEqual(snapshot1.id, "c0def940-9324-11e6-9a56-000f533176b1")
+        self.assertEqual(snapshot2.id, "c2036724-9343-11e6-aef4-000f53315a41")
+        self.assertEqual(snapshot3.id, "d347e033-9343-11e6-9a56-000f533176b1")
 
-class DigitalOceanMockHttp(MockHttpTestCase):
+    def test_create_volume_snapshot(self):
+        volume = self.driver.list_volumes()[0]
+        DigitalOceanMockHttp.type = 'CREATE'
+        snapshot = self.driver.create_volume_snapshot(volume, 'test-snapshot')
+        self.assertEqual(snapshot.id, "c0def940-9324-11e6-9a56-000f533176b1")
+        self.assertEqual(snapshot.name, 'test-snapshot')
+        self.assertEqual(volume.driver, self.driver)
+
+    def test_delete_volume_snapshot(self):
+        volume = self.driver.list_volumes()[0]
+        snapshot = self.driver.list_volume_snapshots(volume)[0]
+        DigitalOceanMockHttp.type = 'DELETE'
+        result = self.driver.delete_volume_snapshot(snapshot)
+        self.assertTrue(result)
+
+    def test_ex_get_node_details(self):
+        node = self.driver.ex_get_node_details('3164444')
+        self.assertEqual(node.name, 'example.com')
+        self.assertEqual(node.public_ips, ['36.123.0.123'])
+        self.assertEqual(node.extra['image']['id'], 12089443)
+        self.assertEqual(node.extra['size_slug'], '8gb')
+        self.assertEqual(len(node.extra['tags']), 2)
+
+    def test_ex_create_floating_ip(self):
+        nyc1 = [r for r in self.driver.list_locations() if r.id == 'nyc1'][0]
+        floating_ip = self.driver.ex_create_floating_ip(nyc1)
+
+        # Note that this is the ID. There is no real ID for a floating IP at
+        # DigitalOcean, but the IP is unique so we can use that instead.
+        self.assertEqual(floating_ip.id, '167.138.123.111')
+        self.assertEqual(floating_ip.ip_address, '167.138.123.111')
+        self.assertEqual(floating_ip.extra['region']['slug'], 'nyc1')
+        # The newly created floating IP reserved to a region is not
+        # associated with any droplet. See the DigitalOcean API docs
+        # how to create a floating IP that is associated with an instance
+        # from the start. This API call creates an unattached IP.
+        self.assertIsNone(floating_ip.node_id)
+
+    def test_ex_delete_floating_ip(self):
+        nyc1 = [r for r in self.driver.list_locations() if r.id == 'nyc1'][0]
+        floating_ip = self.driver.ex_create_floating_ip(nyc1)
+        ret = self.driver.ex_delete_floating_ip(floating_ip)
+
+        # The API returns 204 NO CONTENT if all is well.
+        self.assertTrue(ret)
+
+    def test_floating_ip_can_be_deleted_by_calling_delete_on_floating_ip_object(self):
+        nyc1 = [r for r in self.driver.list_locations() if r.id == 'nyc1'][0]
+        floating_ip = self.driver.ex_create_floating_ip(nyc1)
+        ret = floating_ip.delete()
+
+        self.assertTrue(ret)
+
+    def test_list_floating_ips(self):
+        floating_ips = self.driver.ex_list_floating_ips()
+
+        self.assertEqual(len(floating_ips), 2, 'Wrong floating IPs count')
+
+        floating_ip = floating_ips[0]
+        self.assertEqual(floating_ip.id, '133.166.122.204')
+        self.assertEqual(floating_ip.ip_address, '133.166.122.204')
+        self.assertEqual(floating_ip.extra['region']['slug'], 'ams3')
+        self.assertEqual(84155775, floating_ip.node_id)
+
+    def test_get_floating_ip(self):
+        floating_ip = self.driver.ex_get_floating_ip('133.166.122.204')
+
+        self.assertEqual(floating_ip.id, '133.166.122.204')
+        self.assertEqual(floating_ip.ip_address, '133.166.122.204')
+        self.assertEqual(floating_ip.extra['region']['slug'], 'ams3')
+        self.assertEqual(84155775, floating_ip.node_id)
+
+    def test_ex_attach_floating_ip_to_node(self):
+        node = self.driver.list_nodes()[0]
+        floating_ip = self.driver.ex_get_floating_ip('133.166.122.204')
+
+        ret = self.driver.ex_attach_floating_ip_to_node(node, floating_ip)
+
+        self.assertTrue(ret)
+
+    def test_ex_detach_floating_ip_from_node(self):
+        node = self.driver.list_nodes()[0]
+        floating_ip = self.driver.ex_get_floating_ip('154.138.103.175')
+
+        ret = self.driver.ex_detach_floating_ip_from_node(node, floating_ip)
+
+        self.assertTrue(ret)
+
+
+class DigitalOceanMockHttp(MockHttp):
     fixtures = ComputeFileFixtures('digitalocean_v2')
 
     def _v2_regions(self, method, url, body, headers):
@@ -256,6 +385,10 @@ class DigitalOceanMockHttp(MockHttpTestCase):
         body = self.fixtures.load('list_nodes.json')
         return (httplib.OK, body, {}, httplib.responses[httplib.OK])
 
+    def _v2_droplets_3164444(self, method, url, body, headers):
+        body = self.fixtures.load('list_node.json')
+        return (httplib.OK, body, {}, httplib.responses[httplib.OK])
+
     def _v2_droplets_INVALID_IMAGE(self, method, url, body, headers):
         body = self.fixtures.load('error_invalid_image.json')
         return (httplib.NOT_FOUND, body, {},
@@ -271,6 +404,16 @@ class DigitalOceanMockHttp(MockHttpTestCase):
         return (httplib.NO_CONTENT, body, {},
                 httplib.responses[httplib.NO_CONTENT])
 
+    def _v2_droplets_3164444_actions_KERNELCHANGE(self, method, url, body, headers):
+        # change_kernel
+        body = self.fixtures.load('ex_change_kernel.json')
+        return (httplib.CREATED, body, {}, httplib.responses[httplib.CREATED])
+
+    def _v2_droplets_3164444_actions_ENABLEIPV6(self, method, url, body, headers):
+        # enable_ipv6
+        body = self.fixtures.load('ex_enable_ipv6.json')
+        return (httplib.CREATED, body, {}, httplib.responses[httplib.CREATED])
+
     def _v2_droplets_3164444_actions_RENAME(self, method, url, body, headers):
         # rename_node
         body = self.fixtures.load('ex_rename_node.json')
@@ -303,6 +446,24 @@ class DigitalOceanMockHttp(MockHttpTestCase):
         body = self.fixtures.load('ex_shutdown_node.json')
         return (httplib.CREATED, body, {}, httplib.responses[httplib.CREATED])
 
+    def _v2_droplets_3164444_actions_POWERCYCLE(self, method, url,
+                                                body, headers):
+        # ex_hard_reboot
+        body = self.fixtures.load('ex_hard_reboot.json')
+        return (httplib.CREATED, body, {}, httplib.responses[httplib.OK])
+
+    def _v2_droplets_3164444_actions_REBUILD(self, method, url,
+                                             body, headers):
+        # ex_rebuild_node
+        body = self.fixtures.load('ex_rebuild_node.json')
+        return (httplib.CREATED, body, {}, httplib.responses[httplib.OK])
+
+    def _v2_droplets_3164444_actions_RESIZE(self, method, url,
+                                            body, headers):
+        # ex_resize_node
+        body = self.fixtures.load('ex_resize_node.json')
+        return (httplib.CREATED, body, {}, httplib.responses[httplib.OK])
+
     def _v2_account_keys(self, method, url, body, headers):
         body = self.fixtures.load('list_key_pairs.json')
         return (httplib.OK, body, {}, httplib.responses[httplib.OK])
@@ -353,5 +514,52 @@ class DigitalOceanMockHttp(MockHttpTestCase):
         return (httplib.NO_CONTENT, None, {},
                 httplib.responses[httplib.NO_CONTENT])
 
+    def _v2_volumes_62766883_2c28_11e6_b8e6_000f53306ae1_snapshots_CREATE(
+            self, method, url, body, headers):
+        body = self.fixtures.load('create_volume_snapshot.json')
+        return (httplib.CREATED, body, {}, httplib.responses[httplib.CREATED])
+
+    def _v2_volumes_62766883_2c28_11e6_b8e6_000f53306ae1_snapshots(
+            self, method, url, body, headers):
+        body = self.fixtures.load('list_volume_snapshots.json')
+        return (httplib.OK, body, {}, httplib.responses[httplib.OK])
+
+    def _v2_snapshots_c0def940_9324_11e6_9a56_000f533176b1_DELETE(
+            self, method, url, body, headers):
+        return (httplib.NO_CONTENT, None, {},
+                httplib.responses[httplib.NO_CONTENT])
+
+    def _v2_floating_ips(self, method, url, body, headers):
+        if method == 'POST':
+            body = self.fixtures.load('create_floating_ip.json')
+            return (httplib.OK, body, {}, httplib.responses[httplib.OK])
+        elif method == 'GET':
+            body = self.fixtures.load('list_floating_ips.json')
+            return (httplib.OK, body, {}, httplib.responses[httplib.OK])
+        else:
+            raise NotImplementedError()
+
+    def _v2_floating_ips_167_138_123_111(self, method, url, body, headers):
+        if method == 'DELETE':
+            body = ''
+            return (httplib.NO_CONTENT, body, {}, httplib.responses[httplib.NO_CONTENT])
+        else:
+            raise NotImplementedError()
+
+    def _v2_floating_ips_133_166_122_204_actions(self, method, url, body, headers):
+        if method == 'POST':
+            body = self.fixtures.load('attach_floating_ip.json')
+            return (httplib.CREATED, body, {}, httplib.responses[httplib.CREATED])
+        else:
+            raise NotImplementedError()
+
+    def _v2_floating_ips_154_138_103_175_actions(self, method, url, body, headers):
+        if method == 'POST':
+            body = self.fixtures.load('detach_floating_ip.json')
+            return (httplib.CREATED, body, {}, httplib.responses[httplib.CREATED])
+        else:
+            raise NotImplementedError()
+
+
 if __name__ == '__main__':
     sys.exit(unittest.main())
