@@ -33,19 +33,17 @@ from libcloud.storage.types import ContainerAlreadyExistsError, \
 from libcloud.storage.drivers.atmos import AtmosConnection, AtmosDriver
 from libcloud.storage.drivers.dummy import DummyIterator
 
-from libcloud.test import StorageMockHttp, MockRawResponse
+from libcloud.test import MockHttp, generate_random_data, make_response
 from libcloud.test.file_fixtures import StorageFileFixtures
 
 
 class AtmosTests(unittest.TestCase):
 
     def setUp(self):
-        AtmosDriver.connectionCls.conn_classes = (None, AtmosMockHttp)
-        AtmosDriver.connectionCls.rawResponseCls = AtmosMockRawResponse
+        AtmosDriver.connectionCls.conn_class = AtmosMockHttp
         AtmosDriver.path = ''
         AtmosMockHttp.type = None
         AtmosMockHttp.upload_created = False
-        AtmosMockRawResponse.type = None
         self.driver = AtmosDriver('dummy', base64.b64encode(b('dummy')))
         self._remove_test_file()
 
@@ -189,6 +187,7 @@ class AtmosTests(unittest.TestCase):
             self.fail('Exception was not thrown')
 
     def test_delete_object_success(self):
+        AtmosMockHttp.type = 'DELETE'
         container = Container(name='foo_bar_container', extra={},
                               driver=self.driver)
         obj = Object(name='foo_bar_object', size=1000, hash=None, extra={},
@@ -198,6 +197,7 @@ class AtmosTests(unittest.TestCase):
         self.assertTrue(status)
 
     def test_delete_object_escaped_success(self):
+        AtmosMockHttp.type = 'DELETE'
         container = Container(name='foo & bar_container', extra={},
                               driver=self.driver)
         obj = Object(name='foo & bar_object', size=1000, hash=None, extra={},
@@ -247,7 +247,7 @@ class AtmosTests(unittest.TestCase):
         self.assertTrue(result)
 
     def test_download_object_success_not_found(self):
-        AtmosMockRawResponse.type = 'NOT_FOUND'
+        AtmosMockHttp.type = 'NOT_FOUND'
         container = Container(name='foo_bar_container', extra={},
                               driver=self.driver)
 
@@ -290,12 +290,16 @@ class AtmosTests(unittest.TestCase):
         self.assertTrue(hasattr(stream, '__iter__'))
 
     def test_upload_object_success(self):
-        def upload_file(self, response, file_path, chunked=False,
-                        calculate_hash=True):
-            return True, 'hash343hhash89h932439jsaa89', 1000
+        def upload_file(self, object_name=None, content_type=None,
+                        request_path=None, request_method=None,
+                        headers=None, file_path=None, stream=None):
+            return {'response': make_response(200, headers={'etag': '0cc175b9c0f1b6a831c399e269772661'}),
+                    'bytes_transferred': 1000,
+                    'data_hash': '0cc175b9c0f1b6a831c399e269772661'}
+
+        old_func = AtmosDriver._upload_object
+        AtmosDriver._upload_object = upload_file
 
-        old_func = AtmosDriver._upload_file
-        AtmosDriver._upload_file = upload_file
         path = os.path.abspath(__file__)
         container = Container(name='fbc', extra={}, driver=self)
         object_name = 'ftu'
@@ -305,7 +309,7 @@ class AtmosTests(unittest.TestCase):
         self.assertEqual(obj.name, 'ftu')
         self.assertEqual(obj.size, 1000)
         self.assertTrue('some-value' in obj.meta_data)
-        AtmosDriver._upload_file = old_func
+        AtmosDriver._upload_object = old_func
 
     def test_upload_object_no_content_type(self):
         def no_content_type(name):
@@ -329,13 +333,13 @@ class AtmosTests(unittest.TestCase):
         def dummy_content_type(name):
             return 'application/zip', None
 
-        def send(instance):
-            raise Exception('')
+        def send(self, method, **kwargs):
+            raise LibcloudError('')
 
         old_func1 = libcloud.utils.files.guess_file_mime_type
         libcloud.utils.files.guess_file_mime_type = dummy_content_type
-        old_func2 = AtmosMockHttp.send
-        AtmosMockHttp.send = send
+        old_func2 = AtmosMockHttp.request
+        AtmosMockHttp.request = send
 
         file_path = os.path.abspath(__file__)
         container = Container(name='fbc', extra={}, driver=self)
@@ -352,7 +356,7 @@ class AtmosTests(unittest.TestCase):
                 'Timeout while uploading but an exception was not thrown')
         finally:
             libcloud.utils.files.guess_file_mime_type = old_func1
-            AtmosMockHttp.send = old_func2
+            AtmosMockHttp.request = old_func2
 
     def test_upload_object_nonexistent_file(self):
         def dummy_content_type(name):
@@ -471,7 +475,7 @@ class AtmosTests(unittest.TestCase):
                              b(expected).decode('utf-8'))
 
 
-class AtmosMockHttp(StorageMockHttp, unittest.TestCase):
+class AtmosMockHttp(MockHttp, unittest.TestCase):
     fixtures = StorageFileFixtures('atmos')
     upload_created = False
     upload_stream_created = False
@@ -480,14 +484,15 @@ class AtmosMockHttp(StorageMockHttp, unittest.TestCase):
         unittest.TestCase.__init__(self)
 
         if kwargs.get('host', None) and kwargs.get('port', None):
-            StorageMockHttp.__init__(self, *args, **kwargs)
+            MockHttp.__init__(self, *args, **kwargs)
 
         self._upload_object_via_stream_first_request = True
 
     def runTest(self):
         pass
 
-    def request(self, method, url, body=None, headers=None, raw=False):
+    def request(self, method, url, body=None, headers=None, raw=False,
+                stream=False):
         headers = headers or {}
         parsed = urlparse.urlparse(url)
         if parsed.query.startswith('metadata/'):
@@ -630,11 +635,11 @@ class AtmosMockHttp(StorageMockHttp, unittest.TestCase):
         return (httplib.NOT_FOUND, body, {},
                 httplib.responses[httplib.NOT_FOUND])
 
-    def _rest_namespace_foo_bar_container_foo_bar_object(self, method, url,
-                                                         body, headers):
+    def _rest_namespace_foo_bar_container_foo_bar_object_DELETE(self, method, url,
+                                                                body, headers):
         return (httplib.OK, '', {}, httplib.responses[httplib.OK])
 
-    def _rest_namespace_foo_20_26_20bar_container_foo_20_26_20bar_object(
+    def _rest_namespace_foo_20_26_20bar_container_foo_20_26_20bar_object_DELETE(
         self, method, url,
             body, headers):
         return (httplib.OK, '', {}, httplib.responses[httplib.OK])
@@ -742,29 +747,17 @@ class AtmosMockHttp(StorageMockHttp, unittest.TestCase):
         }
         return (httplib.OK, '', headers, httplib.responses[httplib.OK])
 
-
-class AtmosMockRawResponse(MockRawResponse):
-    fixtures = StorageFileFixtures('atmos')
-
     def _rest_namespace_foo_bar_container_foo_bar_object(self, method, url,
                                                          body, headers):
-        body = self._generate_random_data(1000)
+        body = generate_random_data(1000)
         return (httplib.OK, body, {}, httplib.responses[httplib.OK])
 
     def _rest_namespace_foo_20_26_20bar_container_foo_20_26_20bar_object(
         self, method, url,
             body, headers):
-        body = self._generate_random_data(1000)
+        body = generate_random_data(1000)
         return (httplib.OK, body, {}, httplib.responses[httplib.OK])
 
-    def _rest_namespace_foo_bar_container_foo_bar_object_NOT_FOUND(
-        self, method,
-        url, body,
-            headers):
-        body = self.fixtures.load('not_found.xml')
-        return (httplib.NOT_FOUND, body, {},
-                httplib.responses[httplib.NOT_FOUND])
-
     def _rest_namespace_fbc_ftu(self, method, url, body, headers):
         return (httplib.CREATED, '', {}, httplib.responses[httplib.CREATED])
 
